package pl.primigenius.apdu_service_plugin_example;

public class ApduUtils {

    final protected static char[] hexArray = "0123456789ABCDEF".toCharArray();

    public static byte[] createApduSelectCommand(byte[] apduSelectBase, String aid, byte[] param2) {
        byte[] aidBytes = hexToBytes(aid);
        int totalLength = apduSelectBase.length + aidBytes.length;
        byte[] out = new byte[totalLength];
        System.arraycopy(apduSelectBase, 0, out, 0, 3);
        System.arraycopy(param2, 0, out, 3, 1);
        System.arraycopy(apduSelectBase, 4, out, 4, 1);
        System.arraycopy(aidBytes, 0, out, 5, aidBytes.length);
        System.arraycopy(apduSelectBase, apduSelectBase.length - 1, out, totalLength - 1, 1);
        return out;
    }

    public static String bytesToHex(byte[] bytes) {
        StringBuilder r = new StringBuilder(bytes.length * 2);
        for (byte b : bytes) {
            r.append(hexArray[(b >> 4) & 0xF]);
            r.append(hexArray[(b & 0xF)]);
        }
        return r.toString();
    }

    public static byte[] hexToBytes(String s) {
        final int len = s.length();
        if (len % 2 != 0) {
            throw new IllegalArgumentException("hexBinary needs to be even-length: " + s);
        }
        byte[] out = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            int h = hexToBin(s.charAt(i));
            int l = hexToBin(s.charAt(i + 1));
            if (h == -1 || l == -1) {
                throw new IllegalArgumentException("contains illegal character for hexBinary: " + s);
            }
            out[i / 2] = (byte) (h * 16 + l);
        }
        return out;
    }

    private static int hexToBin(char ch) {
        if ('0' <= ch && ch <= '9') return ch - '0';
        if ('A' <= ch && ch <= 'F') return ch - 'A' + 10;
        if ('a' <= ch && ch <= 'f') return ch - 'a' + 10;
        return -1;
    }

    public static boolean isEqual(byte[] a, byte[] b) {
        if (a == null || b == null || a.length != b.length) {
            return false;
        }
        int result = 0;
        for (int i = 0; i < a.length; i++) {
            result |= a[i] ^ b[i];
        }
        return result == 0;
    }
}
